/*
 * Nasty compatibility functions that don't need to go into the
 * mainline kernel.
 */

#include <linux/smp_lock.h>

#if ((LINUX_VERSION_CODE >= 0x2020f) && (LINUX_VERSION_CODE < 0x20300))
#define ROCKETPORT_HAVE_POLL_WAIT
#endif

#if (LINUX_VERSION_CODE < 131343)  /* 2.1.15 -- XX get correct version */
#define __init
#endif

/*
 * Provide backwards compatibility for kernels prior to 2.1.8.
 */
#if (LINUX_VERSION_CODE < 0x20000)
typedef dev_t kdev_t;
#endif

#if (LINUX_VERSION_CODE < 0x02017E)
static signed long schedule_timeout(signed long timeout)
{
	unsigned long expire;

	expire = timeout + jiffies;

	current->timeout = jiffies + timeout;
	schedule();

	timeout = expire - jiffies;
	return timeout < 0 ? 0 : timeout;
}
#endif

#ifndef time_after
#define time_after(a,b)		((long)(b) - (long)(a) < 0)
#endif

#if (LINUX_VERSION_CODE < 131336)
int copy_from_user(void *to, const void *from_user, unsigned long len)
{
	int	error;

	error = verify_area(VERIFY_READ, from_user, len);
	if (error)
		return len;
	memcpy_fromfs(to, from_user, len);
	return 0;
}

int copy_to_user(void *to_user, const void *from, unsigned long len)
{
	int	error;
	
	error = verify_area(VERIFY_WRITE, to_user, len);
	if (error)
		return len;
	memcpy_tofs(to_user, from, len);
	return 0;
}

static inline int signal_pending(struct task_struct *p)
{
	return ((p->signal & ~p->blocked) != 0);
}
#endif

#ifndef set_current_state
#define set_current_state(state_value)			\
	do { current->state = state_value; } while (0)
#endif

#ifndef DECLARE_WAITQUEUE
#define DECLARE_WAITQUEUE(wait, current) \
	struct wait_queue wait = { current, NULL }
#define init_waitqueue_head(head)	init_waitqueue(head)
#endif

#if defined(ENABLE_PCI) && (LINUX_VERSION_CODE < 0x020163) /* 2.1.99 */
/* For compatibility */
static struct pci_dev *pci_find_slot(unsigned char bus,
				     unsigned char device_fn)
{
	unsigned short		vendor_id, device_id;
	int			ret, error;
	static struct pci_dev	ret_struct;
	unsigned int		port;
	
	error = pcibios_read_config_word(bus, device_fn, PCI_VENDOR_ID,
		&vendor_id);
	ret = pcibios_read_config_word(bus, device_fn, PCI_DEVICE_ID,
		&device_id);
	if (error == 0)
		error = ret;

	if (error) {
		printk("PCI RocketPort error: %s not initializing due to error"
		       "reading configuration space\n",
		       pcibios_strerror(error));
		return(0);
	}
	if (pcibios_read_config_dword(bus, device_fn, PCI_BASE_ADDRESS_0,
				      &port))
		return 0;
	port &= PCI_BASE_ADDRESS_IO_MASK;

	memset(&ret_struct, 0, sizeof(ret_struct));
	ret_struct.device = device_id;
	ret_struct.vendor = vendor_id;
	ret_struct.class = port; /* kludge, kludge, kludge! */
	return &ret_struct;
}
#endif
     
#ifndef PCI_NUM_RESOURCES
unsigned long pci_resource_start(struct pci_dev *dev, int r)
{
	unsigned long port;
	
#if (LINUX_VERSION_CODE >= 0x020163) /* 2.1.99 */
	port = dev->base_address[r];
	if (port & PCI_BASE_ADDRESS_SPACE)
		port &= PCI_BASE_ADDRESS_IO_MASK;
	else
		port &= PCI_BASE_ADDRESS_MEM_MASK;
#else
	port = dev->class;	/* See kludge above */
#endif
	return port;
}
#endif

#if (LINUX_VERSION_CODE < 0x020300) /* 2.3.00 */
int pci_enable_device(struct pci_dev *dev)
{
#ifdef CONFIG_PPC
        u16 pci_command, new_command;
	
	/* The following procedure is neccessary for PowerMac
	 * to enable the PCI card.
	 * This code is stolen from ne2k-pci.c.
	 */
	pci_read_config_word ( dev, PCI_COMMAND, &pci_command );
	new_command = pci_command | PCI_COMMAND_IO;
	if ( pci_command != new_command ) {
		printk ( "The PCI BIOS has not enabled this"
			" RocketPort! Updating PCI command %4.4x->%4.4x\n",
			pci_command, new_command );
		pci_write_config_word ( dev, PCI_COMMAND, new_command );
	} 
#endif	
	return 0;
}
#endif

#if (LINUX_VERSION_CODE < 0x020300) /* 2.3.00 */
#if (defined(CONFIG_M386) || defined(CONFIG_M486) || defined(CONFIG_M586) || \
     defined(CONFIG_M586TSC) || defined(CONFIG_M686))
#define CONFIG_ISA
#define ROCKETPORT_ENABLE_ISA
#endif	     
#endif

#if (LINUX_VERSION_CODE <= 0x2032D)
static void tty_register_devfs  (struct tty_driver *driver, unsigned int flags,
				 unsigned int minor)
{
	return;
}

static void tty_unregister_devfs (struct tty_driver *driver, unsigned minor)
{
	return;
}
#endif

#if (LINUX_VERSION_CODE < 0x020400)
static void del_timer_sync(struct timer_list * timer)
{
	lock_kernel();
	del_timer(timer);
	unlock_kernel();
}
#endif
	
#ifndef mdelay
/* For 2.0 kernels */
#define mdelay(n) (__delay(loops_per_sec/(1000/(n))));
#endif
