/*
 * rocket_int.h --- internal header file for rocket.c
 *
 * Written by Theodore Ts'o, Copyright 1997.
 * Copyright 1997 Comtrol Corporation.  
 * 
 */

#ifndef _ROCKET_INT_H
#define _ROCKET_INT_H


#include <asm/io.h>
#include <asm/byteorder.h>
#include <linux/serial.h>
#include "rocket.h"

#if !defined(AIOPIC) && !defined(UNITY)
   #define AIOPIC
   #define UNITY
#endif

#if (1)

typedef unsigned char Byte_t;
typedef unsigned int ByteIO_t;

typedef unsigned int Word_t;
typedef unsigned int WordIO_t;

typedef unsigned long DWord_t;
typedef unsigned int DWordIO_t;

/*  Number of boards, aiops, ports */
#define MAX_NUMBEROF_BOARDS 8    /*  Max number of boards (ISA + PCI) supported */
#define MAX_AIOPS_PER_BOARD 4    /*  Max number of AIOP's per board */ 
#define MAX_PORTS_PER_AIOP 8     /*  Max number of ports per AIOP */
#define MAX_PORTS_PER_BOARD (MAX_NUMBEROF_BOARDS * MAX_AIOPS_PER_BOARD * MAX_PORTS_PER_AIOP)  /*  Max number of ports per board */

/* Controller level information structure */
typedef struct {
	int CtlNum;                                   //  Controller number (0-n)
	int NumAiop;                                  //  Number of AIOPs on this card
	rocketModel_t rocketModel;                    //  Model information
	int rp_baud_base;                             //  Baud rate fundemental frequency, used in slow/fast clock baud calcs
	int clockPrescale;                            

	int AiopID[MAX_AIOPS_PER_BOARD];              //  If AIOPID_0001 AIOP exists, if -1 it does not
	int AiopNumChan[MAX_AIOPS_PER_BOARD];         //  Number of channels (ports) on the AIOP
	Word_t *AiopIntrBits;                         //  Used when checking AIOP interrupt flags

	/*  ISA IO Addresses */
	ByteIO_t MBaseIO;  //  ISA Mudback IO Addresses
	ByteIO_t MReg1IO;
	ByteIO_t MReg2IO;
	ByteIO_t MReg3IO;
	Byte_t MReg2;
	Byte_t MReg3;

	/*  PCI IO Addresses -  Original PCI cards have a single IO spaces assigned, region 0.  
	    This is the AIOPIC IO space.  UPCI cards have 2 space:  The AIOPIC IO space assigned
	    using region 2 and a "Local Configuration" IO space assigned using region 1. */
	WordIO_t BaseAiopicIOAddr;                    //  Base IO Address of the AIOPIC IO space.  Region 0 on PCI cards, region 2 on uPCI.
	WordIO_t BaseLocalConfigurationIOAddr;        //  Base IO Address of the "Local Configuration" IO space.  uPCI only in region 1.
	WordIO_t IntStatusIOAddr;                     //  IO Address of the interrupt status register in the AIOPIC IO space.
	WordIO_t AiopIO[MAX_AIOPS_PER_BOARD];         //  Base IO address for the AIOP's channel registers (each offset 0x40 from the base IO address)
	ByteIO_t AiopIntChanIO[MAX_AIOPS_PER_BOARD];  //  IO address for the AIOP's Interrupt register (offset 0x39 from the AIOP's base IO address)

	unsigned long pBasePciMemory;                 //  Base PCI memory address (Unity cards only)
} CONTROLLER_T;

#endif


#define Bit(n) (1<<(n))

/* 
 *  Use these macro's to read/write PCI memory space.  This fixes the byte order 
 *  situation and uses the readb/writeb routines.  */
#define READB(addr)  (readb(addr))
#define READW(addr)  (be16_to_cpu(readw(addr)))
#define READL(addr)  (be32_to_cpu(readl(addr)))

#define WRITEB(val, addr) (writeb(val, addr))
#define WRITEW(val, addr) (writew(cpu_to_be16(val), addr))
#define WRITEL(val, addr) (writel(cpu_to_be32(val), addr))

/*  Macros to set/clear a bit in a PCI memory word (short - 16 bits) */
#define SETBITW(addr, bit)  \
do { \
  sval = READW(addr);  \
  WRITEW(sval | bit, addr); \
} while (0)

#define CLEARBITW(addr, bit)  \
do { \
  sval = READW(addr);  \
  WRITEW(sval & ~bit, addr); \
} while (0)

/*  Macros to set/clear a bit in a PCI memory double word (long - 32 bits) */
#define SETBITL(addr, bit)  \
do { \
  lval = READL(addr);  \
  WRITEL(lval | bit, addr); \
} while (0)

#define CLEARBITL(addr, bit)  \
do { \
  lval = READL(addr);  \
  WRITEL(lval & ~bit, addr); \
} while (0)

/***************************************************************/

#if defined(AIOPIC)

static void sSetClockPrescale(CONTROLLER_T * CtlP, int aiopnum, unsigned char prescale);


#endif  //  If AIOPIC


/***************************************************************/

#if defined(UNITY)

static void uSetClockPrescale(CONTROLLER_T *CtlP, int i, int prescale);

#endif  //  IF UNITY


/***************************************************************/

/*  If this is a common driver version */
#if defined(AIOPIC) && defined(UNITY)

static void SetClockPrescale(CONTROLLER_T *CtlP, int i, int prescale);

#endif  //if defined(AIOPIC) && defined(UNITY)


/***************************************************************/

/*  If this is an AIOPIC only driver version */
#if defined(AIOPIC) && !defined(UNITY)

#define SetClockPrescale(CtlP, i, prescale) sSetClockPrescale(CtlP, i, prescale)

#endif  // if defined(AIOPIC) && !defined(UNITY)


/***************************************************************/

/*  If this is a UNITY only driver version */
#if !defined(AIOPIC) && defined(UNITY)

#define SetClockPrescale(CtlP, i, prescale) uSetClockPrescale(CtlP, i, prescale)

#endif  // if !defined(AIOPIC) && defined(UNITY)

/***************************************************************/





/* This is used to move arrays of bytes so byte swapping isn't
 * appropriate.  On Linux 2.3 and above outsw is the same as
 * outsw_ns, but we use the old form for compatibility with
 * old kernels. */
#if  defined(__BIG_ENDIAN) && (LINUX_VERSION_CODE < VERSION_CODE(2,3,0))
#define sOutStrW(port, addr, count) if (count) outsw_ns(port, addr, count)
#define sInStrW(port, addr, count) if (count) insw_ns(port, addr, count)
#else
#define sOutStrW(port, addr, count) if (count) outsw(port, addr, count)
#define sInStrW(port, addr, count) if (count) insw(port, addr, count)
#endif

#ifndef TRUE
#define TRUE 1
#endif

#ifndef FALSE
#define FALSE 0
#endif

#ifndef MIN
#define MIN(a,b)	((a) < (b) ? (a) : (b))
#endif

#define RPORT_MAGIC 0x525001


/*  Comtrol PCI Vendor ID */
#define PCI_VENDOR_ID_RP		0x11fe

/* Major numbers for the Comtrol Rocketport */
#define TTY_ROCKET_MAJOR	46
#define CUA_ROCKET_MAJOR	47

/* tty subtypes */
#define SERIAL_TYPE_NORMAL 1
#define SERIAL_TYPE_CALLOUT 2

/* number of characters left in xmit buffer before we ask for more */
#define WAKEUP_CHARS 256

/*  Clock prescale values */
#define RP_PLUS_PRESCALE      0x12      /*  RocketPort Plus has a fast clock to support 920K baud  */
#define DEFAULT_PRESCALE      0x14      /*  The usual RocketPort prescale value */
#define FAST_CLOCK_PRESCALE   0x19      /*  Slow speed is achieved by using this larger prescale value, ISA only */
#define DEFAULT_BAUD_BASE    460800
    
/* Internal flags used only by the rocketport driver */
#define ROCKET_INITIALIZED	0x80000000	/* Port is active */
#define ROCKET_CLOSING		0x40000000	/* Serial port is closing */
#define ROCKET_NORMAL_ACTIVE	0x20000000	/* Normal port is active */
#define ROCKET_CALLOUT_ACTIVE	0x10000000	/* Callout port is active */

/* Definition of the types in isa_type  */
#define ROCKET_TYPE_NORMAL	0
#define ROCKET_TYPE_MODEM	1
#define ROCKET_TYPE_MODEMII	2
#define ROCKET_TYPE_MODEMIII	3
#define ROCKET_TYPE_PC104       4

#if (0)
/*  Number of boards, aiops, ports */
#define MAX_NUMBEROF_BOARDS 8    /*  Max number of boards (ISA + PCI) supported */
#define MAX_AIOPS_PER_BOARD 4    /*  Max number of AIOP's per board */ 
#define MAX_PORTS_PER_AIOP 8     /*  Max number of ports per AIOP */
#define MAX_PORTS_PER_BOARD (MAX_NUMBEROF_BOARDS * MAX_AIOPS_PER_BOARD * MAX_PORTS_PER_AIOP)  /*  Max number of ports per board */
#endif

/* Bus type ID */
#define	isISA	0
#define	isPCI	1

/* AIOP ID numbers, identifies AIOP type implementing channel */
#define AIOPID_NULL -1		/* no AIOP or channel exists */
#define AIOPID_0001 0x0001	/* AIOP release 1 */

#define NULLDEV -1		/* identifies non-existant device */
#define NULLCTL -1		/* identifies non-existant controller */
#define NULLCTLPTR (CONTROLLER_T *)0	/* identifies non-existant controller */
#define NULLAIOP -1		/* identifies non-existant AIOP */
#define NULLCHAN -1		/* identifies non-existant channel */

/* The size of the xmit buffer is 1 page, or 4096 bytes  */
#define XMIT_BUF_SIZE 4096

/************************************************************************
 Global Register Offsets - Direct Access - Fixed values
************************************************************************/

#define _CMD_REG   0x38		/* Command Register            8    Write */
#define _INT_CHAN  0x39		/* Interrupt Channel Register  8    Read */
#define _INT_MASK  0x3A		/* Interrupt Mask Register     8    Read / Write */
#define _UNUSED    0x3B		/* Unused                      8 */
#define _INDX_ADDR 0x3C		/* Index Register Address      16   Write */
#define _INDX_DATA 0x3E		/* Index Register Data         8/16 Read / Write */

/************************************************************************
 Channel Register Offsets for 1st channel in AIOP - Direct Access
************************************************************************/
#define _TD0       0x00		/* Transmit Data               16   Write */
#define _RD0       0x00		/* Receive Data                16   Read */
#define _CHN_STAT0 0x20		/* Channel Status              8/16 Read / Write */
#define _FIFO_CNT0 0x10		/* Transmit/Receive FIFO Count 16   Read */
#define _INT_ID0   0x30		/* Interrupt Identification    8    Read */

/************************************************************************
 Tx Control Register Offsets - Indexed - External - Fixed
************************************************************************/
#define _TX_ENBLS  0x980	/* Tx Processor Enables Register 8 Read / Write */
#define _TXCMP1    0x988	/* Transmit Compare Value #1     8 Read / Write */
#define _TXCMP2    0x989	/* Transmit Compare Value #2     8 Read / Write */
#define _TXREP1B1  0x98A	/* Tx Replace Value #1 - Byte 1  8 Read / Write */
#define _TXREP1B2  0x98B	/* Tx Replace Value #1 - Byte 2  8 Read / Write */
#define _TXREP2    0x98C	/* Transmit Replace Value #2     8 Read / Write */

/************************************************************************
Memory Controller Register Offsets - Indexed - External - Fixed
************************************************************************/
#define _RX_FIFO    0x000	/* Rx FIFO */
#define _TX_FIFO    0x800	/* Tx FIFO */
#define _RXF_OUTP   0x990	/* Rx FIFO OUT pointer        16 Read / Write */
#define _RXF_INP    0x992	/* Rx FIFO IN pointer         16 Read / Write */
#define _TXF_OUTP   0x994	/* Tx FIFO OUT pointer        8  Read / Write */
#define _TXF_INP    0x995	/* Tx FIFO IN pointer         8  Read / Write */
#define _TXP_CNT    0x996	/* Tx Priority Count          8  Read / Write */
#define _TXP_PNTR   0x997	/* Tx Priority Pointer        8  Read / Write */

#define PRI_PEND    0x80	/* Priority data pending (bit7, Tx pri cnt) */
#define TXFIFO_SIZE 255		/* size of Tx FIFO */
#define RXFIFO_SIZE 1023	/* size of Rx FIFO */

/************************************************************************
Tx Priority Buffer - Indexed - External - Fixed
************************************************************************/
#define _TXP_BUF    0x9C0	/* Tx Priority Buffer  32  Bytes   Read / Write */
#define TXP_SIZE    0x20	/* 32 bytes */

/************************************************************************
Channel Register Offsets - Indexed - Internal - Fixed
************************************************************************/

#define _TX_CTRL    0xFF0	/* Transmit Control               16  Write */
#define _RX_CTRL    0xFF2	/* Receive Control                 8  Write */
#define _BAUD       0xFF4	/* Baud Rate                      16  Write */
#define _CLK_PRE    0xFF6	/* Clock Prescaler                 8  Write */

#define STMBREAK   0x08		/* BREAK */
#define STMFRAME   0x04		/* framing error */
#define STMRCVROVR 0x02		/* receiver over run error */
#define STMPARITY  0x01		/* parity error */
#define STMERROR   (STMBREAK | STMFRAME | STMPARITY)
#define STMBREAKH   0x800	/* BREAK */
#define STMFRAMEH   0x400	/* framing error */
#define STMRCVROVRH 0x200	/* receiver over run error */
#define STMPARITYH  0x100	/* parity error */
#define STMERRORH   (STMBREAKH | STMFRAMEH | STMPARITYH)

#define CTS_ACT   0x20		/* CTS input asserted */
#define DSR_ACT   0x10		/* DSR input asserted */
#define CD_ACT    0x08		/* CD input asserted */
#define TXFIFOMT  0x04		/* Tx FIFO is empty */
#define TXSHRMT   0x02		/* Tx shift register is empty */
#define RDA       0x01		/* Rx data available */
#define DRAINED (TXFIFOMT | TXSHRMT)	/* indicates Tx is drained */

#define STATMODE  0x8000	/* status mode enable bit */
#define RXFOVERFL 0x2000	/* receive FIFO overflow */
#define RX2MATCH  0x1000	/* receive compare byte 2 match */
#define RX1MATCH  0x0800	/* receive compare byte 1 match */
#define RXBREAK   0x0400	/* received BREAK */
#define RXFRAME   0x0200	/* received framing error */
#define RXPARITY  0x0100	/* received parity error */
#define STATERROR (RXBREAK | RXFRAME | RXPARITY)

#define CTSFC_EN  0x80		/* CTS flow control enable bit */
#define RTSTOG_EN 0x40		/* RTS toggle enable bit */
#define TXINT_EN  0x10		/* transmit interrupt enable */
#define STOP2     0x08		/* enable 2 stop bits (0 = 1 stop) */
#define PARITY_EN 0x04		/* enable parity (0 = no parity) */
#define EVEN_PAR  0x02		/* even parity (0 = odd parity) */
#define DATA8BIT  0x01		/* 8 bit data (0 = 7 bit data) */

#define SETBREAK  0x10		/* send break condition (must clear) */
#define LOCALLOOP 0x08		/* local loopback set for test */
#define SET_DTR   0x04		/* assert DTR */
#define SET_RTS   0x02		/* assert RTS */
#define TX_ENABLE 0x01		/* enable transmitter */

#define RTSFC_EN  0x40		/* RTS flow control enable */
#define RXPROC_EN 0x20		/* receive processor enable */
#define TRIG_NO   0x00		/* Rx FIFO trigger level 0 (no trigger) */
#define TRIG_1    0x08		/* trigger level 1 char */
#define TRIG_1_2  0x10		/* trigger level 1/2 */
#define TRIG_7_8  0x18		/* trigger level 7/8 */
#define TRIG_MASK 0x18		/* trigger level mask */
#define SRCINT_EN 0x04		/* special Rx condition interrupt enable */
#define RXINT_EN  0x02		/* Rx interrupt enable */
#define MCINT_EN  0x01		/* modem change interrupt enable */

#define RXF_TRIG  0x20		/* Rx FIFO trigger level interrupt */
#define TXFIFO_MT 0x10		/* Tx FIFO empty interrupt */
#define SRC_INT   0x08		/* special receive condition interrupt */
#define DELTA_CD  0x04		/* CD change interrupt */
#define DELTA_CTS 0x02		/* CTS change interrupt */
#define DELTA_DSR 0x01		/* DSR change interrupt */

#define REP1W2_EN 0x10		/* replace byte 1 with 2 bytes enable */
#define IGN2_EN   0x08		/* ignore byte 2 enable */
#define IGN1_EN   0x04		/* ignore byte 1 enable */
#define COMP2_EN  0x02		/* compare byte 2 enable */
#define COMP1_EN  0x01		/* compare byte 1 enable */

#define RESET_ALL 0x80		/* reset AIOP (all channels) */
#define TXOVERIDE 0x40		/* Transmit software off override */
#define RESETUART 0x20		/* reset channel's UART */
#define RESTXFCNT 0x10		/* reset channel's Tx FIFO count register */
#define RESRXFCNT 0x08		/* reset channel's Rx FIFO count register */

#define INTSTAT0  0x01		/* AIOP 0 interrupt status */
#define INTSTAT1  0x02		/* AIOP 1 interrupt status */
#define INTSTAT2  0x04		/* AIOP 2 interrupt status */
#define INTSTAT3  0x08		/* AIOP 3 interrupt status */

#define INTR_EN   0x08		/* allow interrupts to host */
#define INT_STROB 0x04		/* strobe and clear interrupt line (EOI) */

/**************************************************************************
 MUDBAC remapped for PCI
**************************************************************************/

#define _CFG_INT_PCI  0x40
#define _PCI_INT_FUNC 0x3A

#define PCI_STROB 0x2000	/* bit 13 of int aiop register */
#define INTR_EN_PCI   0x0010	/* allow interrupts to host */

/*
 * Definitions for Universal PCI board registers
 */
#define _PCI_9030_INT_CTRL	0x4c          /* Offsets from BAR1 */
#define _PCI_9030_GPIO_CTRL	0x54
#define PCI_INT_CTRL_AIOP	0x0001
#define PCI_GPIO_CTRL_8PORT	0x4000
#define _PCI_9030_RING_IND	0xc0          /* Offsets from BAR1 */

#define CHAN3_EN  0x08		/* enable AIOP 3 */
#define CHAN2_EN  0x04		/* enable AIOP 2 */
#define CHAN1_EN  0x02		/* enable AIOP 1 */
#define CHAN0_EN  0x01		/* enable AIOP 0 */
#define FREQ_DIS  0x00
#define FREQ_274HZ 0x60
#define FREQ_137HZ 0x50
#define FREQ_69HZ  0x40
#define FREQ_34HZ  0x30
#define FREQ_17HZ  0x20
#define FREQ_9HZ   0x10
#define PERIODIC_ONLY 0x80	/* only PERIODIC interrupt */

#define CHANINT_EN 0x0100	/* flags to enable/disable channel ints */

#define RDATASIZE 72
#define RREGDATASIZE 52

/*
 * AIOP interrupt bits for ISA/PCI boards and UPCI boards.
 */
#define AIOP_INTR_BIT_0		0x0001
#define AIOP_INTR_BIT_1		0x0002
#define AIOP_INTR_BIT_2		0x0004
#define AIOP_INTR_BIT_3		0x0008

#define AIOP_INTR_BITS ( \
	AIOP_INTR_BIT_0 \
	| AIOP_INTR_BIT_1 \
	| AIOP_INTR_BIT_2 \
	| AIOP_INTR_BIT_3)

#define UPCI_AIOP_INTR_BIT_0	0x0004
#define UPCI_AIOP_INTR_BIT_1	0x0020
#define UPCI_AIOP_INTR_BIT_2	0x0100
#define UPCI_AIOP_INTR_BIT_3	0x0800

#define UPCI_AIOP_INTR_BITS ( \
	UPCI_AIOP_INTR_BIT_0 \
	| UPCI_AIOP_INTR_BIT_1 \
	| UPCI_AIOP_INTR_BIT_2 \
	| UPCI_AIOP_INTR_BIT_3)

#if (0)

/* Controller level information structure */
typedef struct {
	int CtlNum;                                   //  Controller number (0-n)
	int NumAiop;                                  //  Number of AIOPs on this card
	rocketModel_t rocketModel;                    //  Model information
	int rp_baud_base;                             //  Baud rate fundemental frequency, used in slow/fast clock baud calcs
	int clockPrescale;                            

	int AiopID[MAX_AIOPS_PER_BOARD];              //  If AIOPID_0001 AIOP exists, if -1 it does not
	int AiopNumChan[MAX_AIOPS_PER_BOARD];         //  Number of channels (ports) on the AIOP
	Word_t *AiopIntrBits;                         //  Used when checking AIOP interrupt flags

	/*  ISA IO Addresses */
	ByteIO_t MBaseIO;  //  ISA Mudback IO Addresses
	ByteIO_t MReg1IO;
	ByteIO_t MReg2IO;
	ByteIO_t MReg3IO;
	Byte_t MReg2;
	Byte_t MReg3;

	/*  PCI IO Addresses -  Original PCI cards have a single IO spaces assigned, region 0.  
	    This is the AIOPIC IO space.  UPCI cards have 2 space:  The AIOPIC IO space assigned
	    using region 2 and a "Local Configuration" IO space assigned using region 1. */
	WordIO_t BaseAiopicIOAddr;                    //  Base IO Address of the AIOPIC IO space.  Region 0 on PCI cards, region 2 on uPCI.
	WordIO_t BaseLocalConfigurationIOAddr;        //  Base IO Address of the "Local Configuration" IO space.  uPCI only in region 1.
	WordIO_t IntStatusIOAddr;                     //  IO Address of the interrupt status register in the AIOPIC IO space.
	WordIO_t AiopIO[MAX_AIOPS_PER_BOARD];         //  Base IO address for the AIOP's channel registers (each offset 0x40 from the base IO address)
	ByteIO_t AiopIntChanIO[MAX_AIOPS_PER_BOARD];  //  IO address for the AIOP's Interrupt register (offset 0x39 from the AIOP's base IO address)

} CONTROLLER_T;

#endif

typedef CONTROLLER_T CONTROLLER_t;

/* Channel level information structure */
typedef struct {
	CONTROLLER_T *CtlP;
	int AiopNum;
	int ChanID;
	int ChanNum;
	int rtsToggle;

	ByteIO_t Cmd;
	ByteIO_t IntChan;
	ByteIO_t IntMask;
	DWordIO_t IndexAddr;
	WordIO_t IndexData;

	WordIO_t TxRxData;
	WordIO_t ChanStat;
	WordIO_t TxRxCount;
	ByteIO_t IntID;

	Word_t TxFIFO;
	Word_t TxFIFOPtrs;
	Word_t RxFIFO;
	Word_t RxFIFOPtrs;
	Word_t TxPrioCnt;
	Word_t TxPrioPtr;
	Word_t TxPrioBuf;

	Byte_t R[RREGDATASIZE];

	Byte_t BaudDiv[MAX_AIOPS_PER_BOARD];
	Byte_t TxControl[MAX_AIOPS_PER_BOARD];
	Byte_t RxControl[MAX_AIOPS_PER_BOARD];
	Byte_t TxEnables[MAX_AIOPS_PER_BOARD];
	Byte_t TxCompare[MAX_AIOPS_PER_BOARD];
	Byte_t TxReplace1[MAX_AIOPS_PER_BOARD];
	Byte_t TxReplace2[MAX_AIOPS_PER_BOARD];
} CHANNEL_T;

typedef CHANNEL_T CHANNEL_t;
typedef CHANNEL_T *CHANPTR_T;


/*  Some static prototypes used in macros here and in rocket.c */
static int cardIsISA(CONTROLLER_T *CtlP);
static int cardIsPC104(CONTROLLER_T *CtlP);
static int cardIsModem(CONTROLLER_T *CtlP);
static int cardIsPCI(CONTROLLER_T *CtlP);
static int cardIsUPCI(CONTROLLER_T *CtlP);
static int cardIsRM2(CONTROLLER_T *CtlP);
static int cardIsRM3(CONTROLLER_T *CtlP);
static int cardIsRocketportPlus(CONTROLLER_T *CtlP);
static int cardIsURPPluswRI(CONTROLLER_T *CtlP);
static int cardIsModem(CONTROLLER_T *CtlP);
static int cardIsUnity(CONTROLLER_T *CtlP);


#define InterfaceModeRS232  0x00
#define InterfaceModeRS422  0x08
#define InterfaceModeRS485  0x10
#define InterfaceModeRS232T 0x18

/*
 * Note!  Normally the Linux I/O macros already take care of
 * byte-swapping the I/O instructions.  However, all accesses using
 * sOutDW aren't really 32-bit accesses, but should be handled in byte
 * order.  Hence the use of the cpu_to_le32() macro to byte-swap
 * things to no-op the byte swapping done by the big-endian outl()
 * instruction.
 */

#ifdef ROCKET_DEBUG_IO
static inline void sOutB(unsigned short port, unsigned char value)
{
#ifdef ROCKET_DEBUG_IO
	printk("sOutB(%x, %x)...", port, value);
#endif
	outb_p(value, port);
}

static inline void sOutW(unsigned short port, unsigned short value)
{
#ifdef ROCKET_DEBUG_IO
	printk("sOutW(%x, %x)...", port, value);
#endif
	outw_p(value, port);
}

static inline void sOutDW(unsigned short port, unsigned long value)
{
#ifdef ROCKET_DEBUG_IO
	printk("sOutDW(%x, %lx)...", port, value);
#endif
	outl_p(cpu_to_le32(value), port);
}

static inline unsigned char sInB(unsigned short port)
{
	return inb_p(port);
}

static inline unsigned short sInW(unsigned short port)
{
	return inw_p(port);
}

#else				/* !ROCKET_DEBUG_IO */
#define sOutB(a, b) outb_p(b, a)
#define sOutW(a, b) outw_p(b, a)
#define sOutDW(port, value) outl_p(cpu_to_le32(value), port)
#define sInB(a) (inb_p(a))
#define sInW(a) (inw_p(a))
#endif

/***************************************************************************
Function: sClrBreak
Purpose:  Stop sending a transmit BREAK signal
Call:     sClrBreak(ChP)
          CHANNEL_T *ChP; Ptr to channel structure
*/
#define sClrBreak(ChP) \
do { \
   (ChP)->TxControl[3] &= ~SETBREAK; \
   sOutDW((ChP)->IndexAddr,*(DWord_t *)&(ChP)->TxControl[0]); \
} while (0)

/***************************************************************************
Function: sClrDTR
Purpose:  Clr the DTR output
Call:     sClrDTR(ChP)
          CHANNEL_T *ChP; Ptr to channel structure
*/
#define sClrDTR(ChP) \
do { \
   (ChP)->TxControl[3] &= ~SET_DTR; \
   sOutDW((ChP)->IndexAddr,*(DWord_t *)&(ChP)->TxControl[0]); \
} while (0)

/***************************************************************************
Function: sClrRTS
Purpose:  Clr the RTS output
Call:     sClrRTS(ChP)
          CHANNEL_T *ChP; Ptr to channel structure
*/
#define sClrRTS(ChP) \
do { \
   if ((ChP)->rtsToggle) break; \
   (ChP)->TxControl[3] &= ~SET_RTS; \
   sOutDW((ChP)->IndexAddr,*(DWord_t *)&(ChP)->TxControl[0]); \
} while (0)

/***************************************************************************
Function: sClrTxXOFF
Purpose:  Clear any existing transmit software flow control off condition
Call:     sClrTxXOFF(ChP)
          CHANNEL_T *ChP; Ptr to channel structure
*/
#define sClrTxXOFF(ChP) \
do { \
   sOutB((ChP)->Cmd,TXOVERIDE | (Byte_t)(ChP)->ChanNum); \
   sOutB((ChP)->Cmd,(Byte_t)(ChP)->ChanNum); \
} while (0)

/***************************************************************************
Function: sCtlNumToCtlPtr
Purpose:  Convert a controller number to controller structure pointer
Call:     sCtlNumToCtlPtr(CtlNum)
          int CtlNum; Controller number
Return:   CONTROLLER_T *: Ptr to controller structure
*/
#define sCtlNumToCtlPtr(CTLNUM) &sController[CTLNUM]

/***************************************************************************
Function: sControllerEOI
Purpose:  Strobe the MUDBAC's End Of Interrupt bit.
Call:     sControllerEOI(CtlP)
          CONTROLLER_T *CtlP; Ptr to controller structure
*/
#define sControllerEOI(CTLP) sOutB((CTLP)->MReg2IO,(CTLP)->MReg2 | INT_STROB)

/***************************************************************************
Function: sPCIControllerEOI
Purpose:  Strobe the PCI End Of Interrupt bit.
          For the UPCI boards, toggle the AIOP interrupt enable bit
	  (this was taken from the Windows driver).
Call:     sPCIControllerEOI(CtlP)
          CONTROLLER_T *CtlP; Ptr to controller structure
*/
#define sPCIControllerEOI(CTLP) \
do { \
    if (cardIsUPCI(CTLP)) { \
	Word_t w = sInW((CTLP)->IntStatusIOAddr); \
	sOutW((CTLP)->IntStatusIOAddr, (w ^ PCI_INT_CTRL_AIOP)); \
	sOutW((CTLP)->IntStatusIOAddr, w); \
    } \
    else { \
	sOutW((CTLP)->IntStatusIOAddr, PCI_STROB); \
    } \
} while (0)

/***************************************************************************
Function: sDisAiop
Purpose:  Disable I/O access to an AIOP
Call:     sDisAiop(CltP)
          CONTROLLER_T *CtlP; Ptr to controller structure
          int AiopNum; Number of AIOP on controller
*/
#define sDisAiop(CTLP,AIOPNUM) \
do { \
   (CTLP)->MReg3 &= sBitMapClrTbl[AIOPNUM]; \
   sOutB((CTLP)->MReg3IO,(CTLP)->MReg3); \
} while (0)

/***************************************************************************
Function: sDisCTSFlowCtl
Purpose:  Disable output flow control using CTS
Call:     sDisCTSFlowCtl(ChP)
          CHANNEL_T *ChP; Ptr to channel structure
*/
#define sDisCTSFlowCtl(ChP) \
do { \
   (ChP)->TxControl[2] &= ~CTSFC_EN; \
   sOutDW((ChP)->IndexAddr,*(DWord_t *)&(ChP)->TxControl[0]); \
} while (0)

/***************************************************************************
Function: sDisIXANY
Purpose:  Disable IXANY Software Flow Control
Call:     sDisIXANY(ChP)
          CHANNEL_T *ChP; Ptr to channel structure
*/
#define sDisIXANY(ChP) \
do { \
   (ChP)->R[0x0e] = 0x86; \
   sOutDW((ChP)->IndexAddr,*(DWord_t *)&(ChP)->R[0x0c]); \
} while (0)

/***************************************************************************
Function: DisParity
Purpose:  Disable parity
Call:     sDisParity(ChP)
          CHANNEL_T *ChP; Ptr to channel structure
Comments: Function sSetParity() can be used in place of functions sEnParity(),
          sDisParity(), sSetOddParity(), and sSetEvenParity().
*/
#define sDisParity(ChP) \
do { \
   (ChP)->TxControl[2] &= ~PARITY_EN; \
   sOutDW((ChP)->IndexAddr,*(DWord_t *)&(ChP)->TxControl[0]); \
} while (0)

/***************************************************************************
Function: sDisRTSToggle
Purpose:  Disable RTS toggle
Call:     sDisRTSToggle(ChP)
          CHANNEL_T *ChP; Ptr to channel structure
*/
#define sDisRTSToggle(ChP) \
do { \
   (ChP)->TxControl[2] &= ~RTSTOG_EN; \
   sOutDW((ChP)->IndexAddr,*(DWord_t *)&(ChP)->TxControl[0]); \
   (ChP)->rtsToggle = 0; \
} while (0)

/***************************************************************************
Function: sDisRxFIFO
Purpose:  Disable Rx FIFO
Call:     sDisRxFIFO(ChP)
          CHANNEL_T *ChP; Ptr to channel structure
*/
#define sDisRxFIFO(ChP) \
do { \
   (ChP)->R[0x32] = 0x0a; \
   sOutDW((ChP)->IndexAddr,*(DWord_t *)&(ChP)->R[0x30]); \
} while (0)

/***************************************************************************
Function: sDisRxStatusMode
Purpose:  Disable the Rx status mode
Call:     sDisRxStatusMode(ChP)
          CHANNEL_T *ChP; Ptr to channel structure
Comments: This takes the channel out of the receive status mode.  All
          subsequent reads of receive data using sReadRxWord() will return
          two data bytes.
*/
#define sDisRxStatusMode(ChP) sOutW((ChP)->ChanStat,0)

/***************************************************************************
Function: sDisTransmit
Purpose:  Disable transmit
Call:     sDisTransmit(ChP)
          CHANNEL_T *ChP; Ptr to channel structure
          This disables movement of Tx data from the Tx FIFO into the 1 byte
          Tx buffer.  Therefore there could be up to a 2 byte latency
          between the time sDisTransmit() is called and the transmit buffer
          and transmit shift register going completely empty.
*/
#define sDisTransmit(ChP) \
do { \
   (ChP)->TxControl[3] &= ~TX_ENABLE; \
   sOutDW((ChP)->IndexAddr,*(DWord_t *)&(ChP)->TxControl[0]); \
} while (0)

/***************************************************************************
Function: sDisTxSoftFlowCtl
Purpose:  Disable Tx Software Flow Control
Call:     sDisTxSoftFlowCtl(ChP)
          CHANNEL_T *ChP; Ptr to channel structure
*/
#define sDisTxSoftFlowCtl(ChP) \
do { \
   (ChP)->R[0x06] = 0x8a; \
   sOutDW((ChP)->IndexAddr,*(DWord_t *)&(ChP)->R[0x04]); \
} while (0)

/***************************************************************************
Function: sEnAiop
Purpose:  Enable I/O access to an AIOP
Call:     sEnAiop(CltP)
          CONTROLLER_T *CtlP; Ptr to controller structure
          int AiopNum; Number of AIOP on controller
*/
#define sEnAiop(CTLP,AIOPNUM) \
do { \
   (CTLP)->MReg3 |= sBitMapSetTbl[AIOPNUM]; \
   sOutB((CTLP)->MReg3IO,(CTLP)->MReg3); \
} while (0)

/***************************************************************************
Function: sEnCTSFlowCtl
Purpose:  Enable output flow control using CTS
Call:     sEnCTSFlowCtl(ChP)
          CHANNEL_T *ChP; Ptr to channel structure
*/
#define sEnCTSFlowCtl(ChP) \
do { \
   (ChP)->TxControl[2] |= CTSFC_EN; \
   sOutDW((ChP)->IndexAddr,*(DWord_t *)&(ChP)->TxControl[0]); \
} while (0)

/***************************************************************************
Function: sEnIXANY
Purpose:  Enable IXANY Software Flow Control
Call:     sEnIXANY(ChP)
          CHANNEL_T *ChP; Ptr to channel structure
*/
#define sEnIXANY(ChP) \
do { \
   (ChP)->R[0x0e] = 0x21; \
   sOutDW((ChP)->IndexAddr,*(DWord_t *)&(ChP)->R[0x0c]); \
} while (0)

/***************************************************************************
Function: EnParity
Purpose:  Enable parity
Call:     sEnParity(ChP)
          CHANNEL_T *ChP; Ptr to channel structure
Comments: Function sSetParity() can be used in place of functions sEnParity(),
          sDisParity(), sSetOddParity(), and sSetEvenParity().

Warnings: Before enabling parity odd or even parity should be chosen using
          functions sSetOddParity() or sSetEvenParity().
*/
#define sEnParity(ChP) \
do { \
   (ChP)->TxControl[2] |= PARITY_EN; \
   sOutDW((ChP)->IndexAddr,*(DWord_t *)&(ChP)->TxControl[0]); \
} while (0)

/***************************************************************************
Function: sEnRTSToggle
Purpose:  Enable RTS toggle
Call:     sEnRTSToggle(ChP)
          CHANNEL_T *ChP; Ptr to channel structure
Comments: This function will disable RTS flow control and clear the RTS
          line to allow operation of RTS toggle.
*/
#define sEnRTSToggle(ChP) \
do { \
   (ChP)->RxControl[2] &= ~RTSFC_EN; \
   sOutDW((ChP)->IndexAddr,*(DWord_t *)&(ChP)->RxControl[0]); \
   (ChP)->TxControl[2] |= RTSTOG_EN; \
   (ChP)->TxControl[3] &= ~SET_RTS; \
   sOutDW((ChP)->IndexAddr,*(DWord_t *)&(ChP)->TxControl[0]); \
   (ChP)->rtsToggle = 1; \
} while (0)

/***************************************************************************
Function: sEnRxFIFO
Purpose:  Enable Rx FIFO
Call:     sEnRxFIFO(ChP)
          CHANNEL_T *ChP; Ptr to channel structure
*/
#define sEnRxFIFO(ChP) \
do { \
   (ChP)->R[0x32] = 0x08; \
   sOutDW((ChP)->IndexAddr,*(DWord_t *)&(ChP)->R[0x30]); \
} while (0)

/***************************************************************************
Function: sEnRxProcessor
Purpose:  Enable the receive processor
Call:     sEnRxProcessor(ChP)
          CHANNEL_T *ChP; Ptr to channel structure
Comments: This function is used to start the receive processor.  When
          the channel is in the reset state the receive processor is not
          running.  This is done to prevent the receive processor from
          executing invalid microcode instructions prior to the
          downloading of the microcode.

Warnings: This function must be called after valid microcode has been
          downloaded to the AIOP, and it must not be called before the
          microcode has been downloaded.
*/
#define sEnRxProcessor(ChP) \
do { \
   (ChP)->RxControl[2] |= RXPROC_EN; \
   sOutDW((ChP)->IndexAddr,*(DWord_t *)&(ChP)->RxControl[0]); \
} while (0)

/***************************************************************************
Function: sEnRxStatusMode
Purpose:  Enable the Rx status mode
Call:     sEnRxStatusMode(ChP)
          CHANNEL_T *ChP; Ptr to channel structure
Comments: This places the channel in the receive status mode.  All subsequent
          reads of receive data using sReadRxWord() will return a data byte
          in the low word and a status byte in the high word.

*/
#define sEnRxStatusMode(ChP) sOutW((ChP)->ChanStat,STATMODE)

/***************************************************************************
Function: sEnTransmit
Purpose:  Enable transmit
Call:     sEnTransmit(ChP)
          CHANNEL_T *ChP; Ptr to channel structure
*/
#define sEnTransmit(ChP) \
do { \
   (ChP)->TxControl[3] |= TX_ENABLE; \
   sOutDW((ChP)->IndexAddr,*(DWord_t *)&(ChP)->TxControl[0]); \
} while (0)

/***************************************************************************
Function: sEnTxSoftFlowCtl
Purpose:  Enable Tx Software Flow Control
Call:     sEnTxSoftFlowCtl(ChP)
          CHANNEL_T *ChP; Ptr to channel structure
*/
#define sEnTxSoftFlowCtl(ChP) \
do { \
   (ChP)->R[0x06] = 0xc5; \
   sOutDW((ChP)->IndexAddr,*(DWord_t *)&(ChP)->R[0x04]); \
} while (0)

/***************************************************************************
Function: sGetAiopIntStatus
Purpose:  Get the AIOP interrupt status
Call:     sGetAiopIntStatus(CtlP,AiopNum)
          CONTROLLER_T *CtlP; Ptr to controller structure
          int AiopNum; AIOP number
Return:   Byte_t: The AIOP interrupt status.  Bits 0 through 7
                         represent channels 0 through 7 respectively.  If a
                         bit is set that channel is interrupting.
*/
#define sGetAiopIntStatus(CTLP,AIOPNUM) sInB((CTLP)->AiopIntChanIO[AIOPNUM])

/***************************************************************************
Function: sGetAiopNumChan
Purpose:  Get the number of channels supported by an AIOP
Call:     sGetAiopNumChan(CtlP,AiopNum)
          CONTROLLER_T *CtlP; Ptr to controller structure
          int AiopNum; AIOP number
Return:   int: The number of channels supported by the AIOP
*/
#define sGetAiopNumChan(CTLP,AIOPNUM) (CTLP)->AiopNumChan[AIOPNUM]

/***************************************************************************
Function: sGetChanIntID
Purpose:  Get a channel's interrupt identification byte
Call:     sGetChanIntID(ChP)
          CHANNEL_T *ChP; Ptr to channel structure
Return:   Byte_t: The channel interrupt ID.  Can be any
             combination of the following flags:
                RXF_TRIG:     Rx FIFO trigger level interrupt
                TXFIFO_MT:    Tx FIFO empty interrupt
                SRC_INT:      Special receive condition interrupt
                DELTA_CD:     CD change interrupt
                DELTA_CTS:    CTS change interrupt
                DELTA_DSR:    DSR change interrupt
*/
#define sGetChanIntID(ChP) (sInB((ChP)->IntID) & (RXF_TRIG | TXFIFO_MT | SRC_INT | DELTA_CD | DELTA_CTS | DELTA_DSR))

/***************************************************************************
Function: sGetChanNum
Purpose:  Get the number of a channel within an AIOP
Call:     sGetChanNum(ChP)
          CHANNEL_T *ChP; Ptr to channel structure
Return:   int: Channel number within AIOP, or NULLCHAN if channel does
               not exist.
*/
#define sGetChanNum(ChP) (ChP)->ChanNum

/***************************************************************************
Function: sGetChanStatus
Purpose:  Get the channel status
Call:     sGetChanStatus(ChP)
          CHANNEL_T *ChP; Ptr to channel structure
Return:   Word_t: The channel status.  Can be any combination of
             the following flags:
                LOW BYTE FLAGS
                CTS_ACT:      CTS input asserted
                DSR_ACT:      DSR input asserted
                CD_ACT:       CD input asserted
                TXFIFOMT:     Tx FIFO is empty
                TXSHRMT:      Tx shift register is empty
                RDA:          Rx data available

                HIGH BYTE FLAGS
                STATMODE:     status mode enable bit
                RXFOVERFL:    receive FIFO overflow
                RX2MATCH:     receive compare byte 2 match
                RX1MATCH:     receive compare byte 1 match
                RXBREAK:      received BREAK
                RXFRAME:      received framing error
                RXPARITY:     received parity error
Warnings: This function will clear the high byte flags in the Channel
          Status Register.
*/
#define sGetChanStatus(ChP) sInW((ChP)->ChanStat)

/***************************************************************************
Function: sGetChanStatusLo
Purpose:  Get the low byte only of the channel status
Call:     sGetChanStatusLo(ChP)
          CHANNEL_T *ChP; Ptr to channel structure
Return:   Byte_t: The channel status low byte.  Can be any combination
             of the following flags:
                CTS_ACT:      CTS input asserted
                DSR_ACT:      DSR input asserted
                CD_ACT:       CD input asserted
                TXFIFOMT:     Tx FIFO is empty
                TXSHRMT:      Tx shift register is empty
                RDA:          Rx data available
*/
#define sGetChanStatusLo(ChP) sInB((ByteIO_t)(ChP)->ChanStat)

/***************************************************************************
Function: sGetControllerIntStatus
Purpose:  Get the controller interrupt status
Call:     sGetControllerIntStatus(CtlP)
          CONTROLLER_T *CtlP; Ptr to controller structure
Return:   Byte_t: The controller interrupt status in the lower 4
                         bits.  Bits 0 through 3 represent AIOP's 0
                         through 3 respectively.  If a bit is set that
                         AIOP is interrupting.  Bits 4 through 7 will
                         always be cleared.
*/
#define sGetControllerIntStatus(CTLP) (sInB((CTLP)->MReg1IO) & 0x0f)

/***************************************************************************
Function: sPCIGetControllerIntStatus
Purpose:  Get the controller interrupt status
Call:     sPCIGetControllerIntStatus(CtlP)
          CONTROLLER_T *CtlP; Ptr to controller structure
Return:   unsigned char: The controller interrupt status in the lower 4
                         bits and bit 4.  Bits 0 through 3 represent AIOP's 0
                         through 3 respectively. Bit 4 is set if the int 
			 was generated from periodic. If a bit is set the
			 AIOP is interrupting.
*/
#define sPCIGetControllerIntStatus(CTLP) \
	(cardIsUPCI(CTLP) ? \
	  (sInW((CTLP)->BaseLocalConfigurationIOAddr + _PCI_9030_GPIO_CTRL) & UPCI_AIOP_INTR_BITS) : \
	  ((sInW((CTLP)->IntStatusIOAddr) >> 8) & AIOP_INTR_BITS))

/***************************************************************************

Function: sGetRxCnt
Purpose:  Get the number of data bytes in the Rx FIFO
Call:     sGetRxCnt(ChP)
          CHANNEL_T *ChP; Ptr to channel structure
Return:   int: The number of data bytes in the Rx FIFO.
Comments: Byte read of count register is required to obtain Rx count.

*/
#define sGetRxCnt(ChP) sInW((ChP)->TxRxCount)

/***************************************************************************
Function: sGetTxCnt
Purpose:  Get the number of data bytes in the Tx FIFO
Call:     sGetTxCnt(ChP)
          CHANNEL_T *ChP; Ptr to channel structure
Return:   Byte_t: The number of data bytes in the Tx FIFO.
Comments: Byte read of count register is required to obtain Tx count.

*/
#define sGetTxCnt(ChP) sInB((ByteIO_t)(ChP)->TxRxCount)

/*****************************************************************************
Function: sGetTxRxDataIO
Purpose:  Get the I/O address of a channel's TxRx Data register
Call:     sGetTxRxDataIO(ChP)
          CHANNEL_T *ChP; Ptr to channel structure
Return:   WordIO_t: I/O address of a channel's TxRx Data register
*/
#define sGetTxRxDataIO(ChP) (ChP)->TxRxData

/***************************************************************************
Function: sInitChanDefaults
Purpose:  Initialize a channel structure to it's default state.
Call:     sInitChanDefaults(ChP)
          CHANNEL_T *ChP; Ptr to the channel structure
Comments: This function must be called once for every channel structure
          that exists before any other SSCI calls can be made.

*/
#define sInitChanDefaults(ChP) \
do { \
   (ChP)->CtlP = NULLCTLPTR; \
   (ChP)->AiopNum = NULLAIOP; \
   (ChP)->ChanID = AIOPID_NULL; \
   (ChP)->ChanNum = NULLCHAN; \
} while (0)

/***************************************************************************
Function: sResetAiopByNum
Purpose:  Reset the AIOP by number
Call:     sResetAiopByNum(CTLP,AIOPNUM)
	CONTROLLER_T CTLP; Ptr to controller structure
	AIOPNUM; AIOP index 
*/
#define sResetAiopByNum(CTLP,AIOPNUM) \
do { \
   sOutB((CTLP)->AiopIO[(AIOPNUM)]+_CMD_REG,RESET_ALL); \
   sOutB((CTLP)->AiopIO[(AIOPNUM)]+_CMD_REG,0x0); \
} while (0)

/***************************************************************************
Function: sSendBreak
Purpose:  Send a transmit BREAK signal
Call:     sSendBreak(ChP)
          CHANNEL_T *ChP; Ptr to channel structure
*/
#define sSendBreak(ChP) \
do { \
   (ChP)->TxControl[3] |= SETBREAK; \
   sOutDW((ChP)->IndexAddr,*(DWord_t *)&(ChP)->TxControl[0]); \
} while (0)

/***************************************************************************
Function: sSetBaud
Purpose:  Set baud rate
Call:     sSetBaud(ChP,Divisor)
          CHANNEL_T *ChP; Ptr to channel structure
          Word_t Divisor; 16 bit baud rate divisor for channel
*/
#define sSetBaud(ChP,DIVISOR) \
do { \
   (ChP)->BaudDiv[2] = (Byte_t)(DIVISOR); \
   (ChP)->BaudDiv[3] = (Byte_t)((DIVISOR) >> 8); \
   sOutDW((ChP)->IndexAddr,*(DWord_t *)&(ChP)->BaudDiv[0]); \
} while (0)

/***************************************************************************
Function: sSetData7
Purpose:  Set data bits to 7
Call:     sSetData7(ChP)
          CHANNEL_T *ChP; Ptr to channel structure
*/
#define sSetData7(ChP) \
do { \
   (ChP)->TxControl[2] &= ~DATA8BIT; \
   sOutDW((ChP)->IndexAddr,*(DWord_t *)&(ChP)->TxControl[0]); \
} while (0)

/***************************************************************************
Function: sSetData8
Purpose:  Set data bits to 8
Call:     sSetData8(ChP)
          CHANNEL_T *ChP; Ptr to channel structure
*/
#define sSetData8(ChP) \
do { \
   (ChP)->TxControl[2] |= DATA8BIT; \
   sOutDW((ChP)->IndexAddr,*(DWord_t *)&(ChP)->TxControl[0]); \
} while (0)

/***************************************************************************
Function: sSetDTR
Purpose:  Set the DTR output
Call:     sSetDTR(ChP)
          CHANNEL_T *ChP; Ptr to channel structure
*/
#define sSetDTR(ChP) \
do { \
   (ChP)->TxControl[3] |= SET_DTR; \
   sOutDW((ChP)->IndexAddr,*(DWord_t *)&(ChP)->TxControl[0]); \
} while (0)

/***************************************************************************
Function: sSetEvenParity
Purpose:  Set even parity
Call:     sSetEvenParity(ChP)
          CHANNEL_T *ChP; Ptr to channel structure
Comments: Function sSetParity() can be used in place of functions sEnParity(),
          sDisParity(), sSetOddParity(), and sSetEvenParity().

Warnings: This function has no effect unless parity is enabled with function
          sEnParity().
*/
#define sSetEvenParity(ChP) \
do { \
   (ChP)->TxControl[2] |= EVEN_PAR; \
   sOutDW((ChP)->IndexAddr,*(DWord_t *)&(ChP)->TxControl[0]); \
} while (0)

/***************************************************************************
Function: sSetOddParity
Purpose:  Set odd parity
Call:     sSetOddParity(ChP)
          CHANNEL_T *ChP; Ptr to channel structure
Comments: Function sSetParity() can be used in place of functions sEnParity(),
          sDisParity(), sSetOddParity(), and sSetEvenParity().

Warnings: This function has no effect unless parity is enabled with function
          sEnParity().
*/
#define sSetOddParity(ChP) \
do { \
   (ChP)->TxControl[2] &= ~EVEN_PAR; \
   sOutDW((ChP)->IndexAddr,*(DWord_t *)&(ChP)->TxControl[0]); \
} while (0)

/***************************************************************************
Function: sSetRTS
Purpose:  Set the RTS output
Call:     sSetRTS(ChP)
          CHANNEL_T *ChP; Ptr to channel structure
*/
#define sSetRTS(ChP) \
do { \
   if ((ChP)->rtsToggle) break; \
   (ChP)->TxControl[3] |= SET_RTS; \
   sOutDW((ChP)->IndexAddr,*(DWord_t *)&(ChP)->TxControl[0]); \
} while (0)

/***************************************************************************
Function: sSetRxTrigger
Purpose:  Set the Rx FIFO trigger level
Call:     sSetRxProcessor(ChP,Level)
          CHANNEL_T *ChP; Ptr to channel structure
          Byte_t Level; Number of characters in Rx FIFO at which the
             interrupt will be generated.  Can be any of the following flags:

             TRIG_NO:   no trigger
             TRIG_1:    1 character in FIFO
             TRIG_1_2:  FIFO 1/2 full
             TRIG_7_8:  FIFO 7/8 full
Comments: An interrupt will be generated when the trigger level is reached
          only if function sEnInterrupt() has been called with flag
          RXINT_EN set.  The RXF_TRIG flag in the Interrupt Idenfification
          register will be set whenever the trigger level is reached
          regardless of the setting of RXINT_EN.

*/
#define sSetRxTrigger(ChP,LEVEL) \
do { \
   (ChP)->RxControl[2] &= ~TRIG_MASK; \
   (ChP)->RxControl[2] |= LEVEL; \
   sOutDW((ChP)->IndexAddr,*(DWord_t *)&(ChP)->RxControl[0]); \
} while (0)

/***************************************************************************
Function: sSetStop1
Purpose:  Set stop bits to 1
Call:     sSetStop1(ChP)
          CHANNEL_T *ChP; Ptr to channel structure
*/
#define sSetStop1(ChP) \
do { \
   (ChP)->TxControl[2] &= ~STOP2; \
   sOutDW((ChP)->IndexAddr,*(DWord_t *)&(ChP)->TxControl[0]); \
} while (0)

/***************************************************************************
Function: sSetStop2
Purpose:  Set stop bits to 2
Call:     sSetStop2(ChP)
          CHANNEL_T *ChP; Ptr to channel structure
*/
#define sSetStop2(ChP) \
do { \
   (ChP)->TxControl[2] |= STOP2; \
   sOutDW((ChP)->IndexAddr,*(DWord_t *)&(ChP)->TxControl[0]); \
} while (0)

/***************************************************************************
Function: sSetTxXOFFChar
Purpose:  Set the Tx XOFF flow control character
Call:     sSetTxXOFFChar(ChP,Ch)
          CHANNEL_T *ChP; Ptr to channel structure
          Byte_t Ch; The value to set the Tx XOFF character to
*/
#define sSetTxXOFFChar(ChP,CH) \
do { \
   (ChP)->R[0x07] = (CH); \
   sOutDW((ChP)->IndexAddr,*(DWord_t *)&(ChP)->R[0x04]); \
} while (0)

/***************************************************************************
Function: sSetTxXONChar
Purpose:  Set the Tx XON flow control character
Call:     sSetTxXONChar(ChP,Ch)
          CHANNEL_T *ChP; Ptr to channel structure
          Byte_t Ch; The value to set the Tx XON character to
*/
#define sSetTxXONChar(ChP,CH) \
do { \
   (ChP)->R[0x0b] = (CH); \
   sOutDW((ChP)->IndexAddr,*(DWord_t *)&(ChP)->R[0x08]); \
} while (0)

/***************************************************************************
Function: sStartRxProcessor
Purpose:  Start a channel's receive processor
Call:     sStartRxProcessor(ChP)
          CHANNEL_T *ChP; Ptr to channel structure
Comments: This function is used to start a Rx processor after it was
          stopped with sStopRxProcessor() or sStopSWInFlowCtl().  It
          will restart both the Rx processor and software input flow control.

*/
#define sStartRxProcessor(ChP) sOutDW((ChP)->IndexAddr,*(DWord_t *)&(ChP)->R[0])

/***************************************************************************
Function: sWriteTxByte
Purpose:  Write a transmit data byte to a channel.
          ByteIO_t io: Channel transmit register I/O address.  This can
                           be obtained with sGetTxRxDataIO().
          Byte_t Data; The transmit data byte.
Warnings: This function writes the data byte without checking to see if
          sMaxTxSize is exceeded in the Tx FIFO.
*/
#define sWriteTxByte(IO,DATA) sOutB(IO,DATA)

/*
 * Begin Linux specific definitions for the Rocketport driver
 *
 * This code is Copyright Theodore Ts'o, 1995-1997
 */

struct r_port {
	int magic;
	int line;
	int flags;
	int count;
	int blocked_open;
	struct tty_struct *tty;
	unsigned int board:3;
	unsigned int aiop:2;
	unsigned int chan:3;
	CONTROLLER_t *CtlP;
	CHANNEL_t channel;
	int closing_wait;
	int close_delay;
	int intmask;
	int xmit_fifo_room;	/* room in xmit fifo */
	unsigned char *xmit_buf;
	int xmit_head;
	int xmit_tail;
	int xmit_cnt;
	int session;
	int pgrp;
	int cd_status;
	int ignore_status_mask;
	int read_status_mask;
	int cps;
	struct termios normal_termios;
	struct termios callout_termios;

#ifdef DECLARE_WAITQUEUE
	wait_queue_head_t open_wait;
	wait_queue_head_t close_wait;
	wait_queue_head_t delta_msr_wait;
#else
	struct wait_queue *open_wait;
	struct wait_queue *close_wait;
	struct wait_queue *delta_msr_wait;
#endif
        struct async_icount icount;  //  Keeps count of MSR line changes
	int am_delta_msr_waiting;    //  Flag, indicating a process has called TIOCMIWAIT and is sleeping
	unsigned long last_msr;      //  The last MSR values read

	spinlock_t slock;
	struct semaphore write_sem;
};

/*  Taking care of some kernel incompatibilities... */
#if LINUX_VERSION_CODE > VERSION_CODE(2,5,68)

#define TTY_GET_LINE(t) t->index

#define TTY_DRIVER_MINOR_START(t) t->driver->minor_start
#define TTY_DRIVER_SUBTYPE(t) t->driver->subtype
#define TTY_DRIVER_NAME(t) t->driver->name
#define TTY_DRIVER_NAME_BASE(t) t->driver->name_base
#define TTY_DRIVER_FLUSH_BUFFER_EXISTS(t) t->driver->flush_buffer
#define TTY_DRIVER_FLUSH_BUFFER(t) t->driver->flush_buffer(t)

#else

#define TTY_GET_LINE(t) minor(t->device) - TTY_DRIVER_MINOR_START(t)

#define TTY_DRIVER_MINOR_START(t) t->driver.minor_start
#define TTY_DRIVER_SUBTYPE(t) t->driver.subtype
#define TTY_DRIVER_NAME(t) t->driver.name
#define TTY_DRIVER_NAME_BASE(t) t->driver.name_base
#define TTY_DRIVER_FLUSH_BUFFER_EXISTS(t) t->driver.flush_buffer
#define TTY_DRIVER_FLUSH_BUFFER(t) t->driver.flush_buffer(t)

#endif //  LINUX_VERSION > 2.5


/********************************************************************
 ********************************************************************
 ***      UNITY HARDWARE DEFINES     ********************************/

/*  Overall memory map */
#define PORT_REGS_BASE_ADDR        0x0000
#define AIOP_INT_STATUS            0x0410
#define AIOP_INT_MASK              0x0414

#define I2C_CONTROLLER_BASE        0x0500
#define DMA_CONTROLLER_BASE        0x0600

#define ASIC_PERIODIC_INT_STATUS   0x0700
#define ASIC_INT_STATUS            0x0700
#define ASIC_INT_ENABLE            0x0702
#define ASIC_CONFIGURATION         0x0704
#define ASIC_CLOCK_PRESCALER       0x0708
#define ASIC_GLOBAL_COMMAND        0x070C
#define ASIC_MODEM_RESET           0x0710

#define PORT_REGS_BASE_ADDR_CHIP2  0x1000
#define AIOP_INT_STATUS_CHIP2      0x1410
#define AIOP_INT_MASK_CHIP2        0x1414

/*  Port Register Offsets */
#define RCV_DATA_BLOCK             0x0000
#define XMIT_DATA_BLOCK            0x0000
#define RCV_DATA_WORD              0x0004
#define XMIT_DATA_WORD             0x0004
#define RCV_DATA_BYTE_STATUS       0x0008
#define XMIT_DATA_BYTE             0x0008
#define RCV_FIFO_COUNT             0x000C
#define XMIT_FIFO_COUNT            0x000E
#define INT_ID_CHAN_STATUS         0x0010
#define CHAN_STATUS                0x0010
#define INT_ID                     0x0012
#define XMIT_CTRL_RCV_CTRL         0x0014
#define RCV_CTRL                   0x0014
#define XMIT_CTRL                  0x0016
#define UART_CTRL                  0x0018
#define BAUD_RATE_GENERATOR        0x001C
#define INTERCHAR_RELOAD_VAL       0x0020
#define PRIORITY_XMIT_FIFO         0x0024

/*  Bit defines for RCV_DATA_BYTE_STATUS (offset 0x08) */
#define RCV_BYTE_PARITY_ERROR      Bit(8)
#define RCV_BYTE_RCV_OVERRUN       Bit(9)
#define RCV_BYTE_FRAMING_ERROR     Bit(10)
#define RCV_BYTE_BREAK_DETECTED    Bit(11)
#define RCV_BYTE_ADDRESS_BYTE      Bit(12)
#define RCV_BYTE_TIMEOUT_DETECTED  Bit(13)

/*  Bit defines for INT_ID_CHAN_STATUS (offset 0x10) */
/*  Bit defines for CHAN_STATUS (offset 0x10) */
#define RCV_DATA_AVAIL             Bit(0)
#define XMIT_SHIFT_REG_EMPTY       Bit(1)
#define XMIT_FIFO_EMPTY            Bit(2)
#define CD_ACTIVE                  Bit(3)
#define DSR_ACTIVE                 Bit(4)
#define CTS_ACTIVE                 Bit(5)
#define RI_ACTIVE                  Bit(6)
#define CHAN_STATUS_PARITY_ERROR   Bit(8)
#define CHAN_STATUS_FRAMING_ERROR  Bit(9)
#define CHAN_STATUS_BREAK_DETECTED Bit(10)
#define RCV_MATCH1                 Bit(11)
#define RCV_MATCH2                 Bit(12)
#define CHAN_STATUS_RCV_OVERRUN    Bit(13)
#define PRIORITY_BUFFER_FULL       Bit(14)
#define UART_ERROR_MASK            ((CHAN_STATUS_RCV_OVERRUN) | (CHAN_STATUS_BREAK_DETECTED) | (CHAN_STATUS_FRAMING_ERROR) | (CHAN_STATUS_PARITY_ERROR))

/*  Bit defines for INT_ID (offset 0x12) */
#if (1)  //  If using 16 bit access, INT_ID flags start at bit 0
#define INT_DSR_CHANGE             Bit(0)
#define INT_CTS_CHANGE             Bit(1)
#define INT_CD_CHANGE              Bit(2)
#define INT_SPECIAL_RECEIVE        Bit(3)
#define INT_XMIT_FIFO_EMPTY        Bit(4)
#define INT_RCV_FIFO_TRIGGER       Bit(5)
#define INT_RI_CHANGE              Bit(6)
#define INT_RCV_DATA_RDY           Bit(7)
#define INT_ADDR_RCVD              Bit(8)
#define INT_XMIT_SHIFTREG_EMPTY    Bit(9)
#define INT_XMIT_FIFO_TRIGGER      Bit(10)
#define INT_TIMEOUT_DETECTED       Bit(11)
#else
#define INT_DSR_CHANGE             Bit(16)
#define INT_CTS_CHANGE             Bit(17)
#define INT_CD_CHANGE              Bit(18)
#define INT_SPECIAL_RECEIVE        Bit(19)
#define INT_XMIT_FIFO_EMPTY        Bit(20)
#define INT_RCV_FIFO_TRIGGER       Bit(21)
#define INT_RI_CHANGE              Bit(22)
#define INT_RCV_DATA_RDY           Bit(23)
#define INT_ADDR_RCVD              Bit(24)
#define INT_XMIT_SHIFTREG_EMPTY    Bit(25)
#define INT_XMIT_FIFO_TRIGGER      Bit(26)
#define INT_TIMEOUT_DETECTED       Bit(27)
#endif // 16 bit INT_ID/CHAN_STATUS

/*  Bit defines for XMIT_CTRL_RCV_CTRL (offset 0x14) */
/*  Bit defines for RCV_CTRL (offset 0x14) */
#define MDM_CHANGE_INT_EN          Bit(0)
#define RCV_INT_EN                 Bit(1)
#define SPECIAL_RCV_INT_EN         Bit(2)
#define RCV_FIFO_TRIGGER0          Bit(3)
#define RCV_FIFO_TRIGGER1          Bit(4)
#define RCV_PROC_EN                Bit(5)
#define RTS_FLOW_EN                Bit(6)
#define DTR_FLOW_EN                Bit(7)
#define ONE_STOP_BIT_EN            Bit(8)
#define RCV_DATA_RDY_INT_EN        Bit(9)
#define ADDRESS_INT_EN             Bit(10)
#define INTERCHAR_INT_EN           Bit(11)
#define INTERCHAR_STATUS_EN        Bit(12)

/*  Bit defines for XMIT_CTRL (offset 0x16)  */
#if (1)  //  If using 16 bit access, XMIT_CTRL flags start at bit 0
#define XMIT_FIFO_TRIGGER1         Bit(0)
#define XMIT_FIFO_TRIGGER2         Bit(1)
#define DSR_FLOW_EN                Bit(2)
#define XMIT_SHIFTREG_INT_EN       Bit(3)
#define XMIT_FIFO_INT_EN           Bit(4)
#define RTS_TOGGLE_POLARITY        Bit(5)
#define RTS_TOGGLE                 Bit(6)
#define CTS_FLOW_EN                Bit(7)
#define XMIT_EN                    Bit(8)
#define RTS_ACTIVE                 Bit(9)
#define DTR_ACTIVE                 Bit(10)
#define LOCAL_LOOPBACK             Bit(11)
#define BREAK_ON                   Bit(12)
#define SET_PARITY                 Bit(13)
#define EVEN_PARITY                Bit(14)
#define PARITY_ON                  Bit(15)
#else
#define XMIT_FIFO_TRIGGER1         Bit(16)
#define XMIT_FIFO_TRIGGER2         Bit(17)
#define DSR_FLOW_EN                Bit(18)
#define XMIT_SHIFTREG_INT_EN       Bit(19)
#define XMIT_FIFO_INT_EN           Bit(20)
#define RTS_TOGGLE_POLARITY        Bit(21)
#define RTS_TOGGLE                 Bit(22)
#define CTS_FLOW_EN                Bit(23)
#define XMIT_EN                    Bit(24)
#define RTS_ACTIVE                 Bit(25)
#define DTR_ACTIVE                 Bit(26)
#define LOCAL_LOOPBACK             Bit(27)
#define BREAK_ON                   Bit(28)
#define SET_PARITY                 Bit(29)
#define EVEN_PARITY                Bit(30)
#define PARITY_ON                  Bit(31)
#endif // 16 bit XMIT/RCV CTRL

/*  Bit defines for UART_CTRL (offset 0x18) */
#define INTERFACE_SELECT0          Bit(0)
#define INTERFACE_SELECT1          Bit(1)
#define INTERFACE_SELECT2          Bit(2)
#define FLUSH_RCV_FIFO             Bit(3)
#define FLUSH_XMIT_FIFO            Bit(4)
#define RESET_CHANNEL              Bit(5)
#define RCV_PROC_XMIT_EN           Bit(6)
#define FLUSH_PRIORITY_FIFO        Bit(7)
#define DATA_BITS0                 Bit(8)
#define DATA_BITS1                 Bit(9)
#define STOP_BITS                  Bit(10)

/*  Bit defines for ASIC_PERIODIC_INT_STATUS (offset 0x700) */
/*  Bit defines for ASIC_INT_STATUS (offset 0x700) */
#define INT_ACTIVE                 Bit(0)
#define DMA_INT_ACTIVE             Bit(1)
#define AIOP0_INT_ACTIVE           Bit(4)
#define AIOP1_INT_ACTIVE           Bit(5)
#define AIOP2_INT_ACTIVE           Bit(6)
#define AIOP3_INT_ACTIVE           Bit(7)
#define PERIODIC_INT_ACTIVE        Bit(8)

/*  Bit defines for ASIC_INT_ENABLE (offset 0x702) */
#if (1)  //  If using 16 bit access, GLOBALINT_EN flags start at bit 0
#define GLOBAL_INT_EN              Bit(0)
#define DMA_INT_EN                 Bit(1)
#define AIOP_INT_EN                Bit(4)
#define PERIODIC_TIMER0            Bit(8)
#define PERIODIC_TIMER1            Bit(9)
#define PERIODIC_TIMER2            Bit(10)
#else
#define GLOBAL_INT_EN              Bit(16)
#define DMA_INT_EN                 Bit(17)
#define AIOP_INT_EN                Bit(20)
#define PERIODIC_TIMER0            Bit(24)
#define PERIODIC_TIMER1            Bit(25)
#define PERIODIC_TIMER2            Bit(26)
#endif  //  ASIC_PERIODIC_INT_STATUS 16 bit XMIT/RCV CTRL

/*  Bit defines for ASIC_CONFIGURATION (offset 0x704) */
#define BUS_MODE0                  Bit(0)
#define BUS_MODE1                  Bit(1)
#define TDM_MODE                   Bit(2)
#define TDM_IF                     Bit(3)
#define EIGHT_PORT_IF              Bit(4)
#define MODEM_RESET_SELECT         Bit(5)
#define DOUBLE_DATA                Bit(6)
#define TDM_CLK_SPEED0             Bit(8)
#define TDM_CLK_SPEED1             Bit(9)
#define TDM_TEST                   Bit(10)
#define SLOW_CLOCK                 Bit(11)

/*  Bit defines for the Periodic Timer bites of the ASIC_INT_ENABLE (16 bit access) */
#define PTIMER_MASK                (Bit(10) | Bit(9) | Bit(8))
#define PTIMER_DISABLED            (0x00)
#define PTIMER_8_789               (Bit(8))
#define PTIMER_17_578              (Bit(9))
#define PTIMER_35_156              (Bit(9) | Bit(8))
#define PTIMER_70_3125             (Bit(10))
#define PTIMER_140_625             (Bit(10) | Bit(8))
#define PTIMER_281_25              (Bit(10) | Bit(9))
#define PTIMER_562_5               (Bit(10) | Bit(9) | Bit(8))

/*  Bit defines for the XMIT FIFO Trigger level of tye XMIT_CTRL register (16 bit access) */
#define UXMIT_FIFO_TRIG_MASK       (Bit(1) | Bit(0))
#define UXMIT_FIFO_TRIG_DISABLED   (0x00)
#define UXMIT_FIFO_TRIG_1_8        (Bit(0))
#define UXMIT_FIFO_TRIG_1_4        (Bit(1))
#define UXMIT_FIFO_TRIG_1_2        (Bit(1) | Bit(0))

/*  Bit defines for the RCV FIFO Trigger level of tye RCV_CTRL register */
#define URCV_FIFO_TRIG_MASK       (Bit(4) | Bit(3))
#define URCV_FIFO_TRIG_DISABLED   (0x00)
#define URCV_FIFO_TRIG_1_8        (Bit(3))
#define URCV_FIFO_TRIG_1_4        (Bit(4))
#define URCV_FIFO_TRIG_1_2        (Bit(4) | Bit(3))

/********************************  JUNK  **************************/
#define PORT0_BASE_ADDR            0x0000
#define PORT1_BASE_ADDR            0x0040
#define PORT2_BASE_ADDR            0x0080
#define PORT3_BASE_ADDR            0x00C0
#define PORT4_BASE_ADDR            0x0100
#define PORT5_BASE_ADDR            0x0140
#define PORT6_BASE_ADDR            0x0180
#define PORT7_BASE_ADDR            0x01C0

#define PORT8_BASE_ADDR            0x0200
#define PORT9_BASE_ADDR            0x0240
#define PORT10_BASE_ADDR           0x0280
#define PORT11_BASE_ADDR           0x02C0
#define PORT12_BASE_ADDR           0x0300
#define PORT13_BASE_ADDR           0x0340
#define PORT14_BASE_ADDR           0x0380
#define PORT15_BASE_ADDR           0x03C0

#define PORT16_BASE_ADDR    0x1000
#define PORT17_BASE_ADDR    0x1040
#define PORT18_BASE_ADDR    0x1080
#define PORT19_BASE_ADDR    0x10C0
#define PORT20_BASE_ADDR    0x1100
#define PORT21_BASE_ADDR    0x1140
#define PORT22_BASE_ADDR    0x1180
#define PORT23_BASE_ADDR    0x11C0

#define PORT24_BASE_ADDR    0x1200
#define PORT25_BASE_ADDR   0x1240
#define PORT26_BASE_ADDR   0x1280
#define PORT27_BASE_ADDR   0x12C0
#define PORT28_BASE_ADDR   0x1300
#define PORT29_BASE_ADDR   0x1340
#define PORT30_BASE_ADDR   0x1380
#define PORT31_BASE_ADDR   0x13C0

#define AIOP_CTL_SIZE 4

/*  Trigger defines, these taken from Linux driver, are common  */
#define TRIG_NO   0x00		/* Rx FIFO trigger level 0 (no trigger) */
#define TRIG_1    0x08		/* trigger level 1 char */
#define TRIG_1_2  0x10		/* trigger level 1/2 */
#define TRIG_7_8  0x18		/* trigger level 7/8 */
#define TRIG_MASK 0x18		/* trigger level mask */

/*  Data bit defines, unique to UNITY.  */
#define DATABITS_MASK (Bit(9) | Bit(8))	
#define DATABITS_5    (0x00)
#define DATABITS_6    (Bit(8))
#define DATABITS_7    (Bit(9))
#define DATABITS_8    (Bit(9) | Bit(8))

/*  RS mode bit defines, unique to UNITY.  */
#define URS_MASK             (Bit(2) | Bit(1) | Bit(0))	
#define URS_DISABLED         (0x00)
#define URS_RS232            (Bit(0))
#define URS_RS422            (Bit(1))
#define URS_RS485_FD_MASTER  (Bit(1))
#define URS_RS485_FD_SLAVE   (Bit(1) | Bit(0))
#define URS_RS485_HD         (Bit(2))
#define URS_RS485_TEST       (Bit(2) | Bit(0))
#define URS_RESERVED         (Bit(2) | Bit(1))
#define URS_DEBUG            (Bit(2) | Bit(1) | Bit(0))

#endif //  #ifndef _ROCKET_INT_H
