#!/bin/sh
#
# This script is responsible for installing and uninstalling the 
# Comtrol Rocketport driver 
#

if [ `whoami` != root ]; then
	echo "You must be root to install or uninstall the device driver!"
	exit 1
fi

OBJS=rocket.o

if [ "$1"x = "uninstallx" ]; then
	find /lib/modules -name rocket.o -exec rm {} \;
	find /lib/modules -name mod_rocket.o -exec rm {} \;
	find /etc/rc.d -lname \*/init.d/rocket -exec rm {} \;
	for i in /etc/init.d /etc/rc.d/init.d 
	do
	    test -d $i && find $i -name rocket -exec rm {} \;
	done
	exit 0
fi

# Install a copy of the driver in the version independent location
#if [ ! -d /lib/modules ]; then 
#	mkdir -p /lib/modules
#fi
#
#cp $OBJS /lib/modules

# Install a copy of the driver in a kernel version specific location
#  LINUX_SRC contains the path to the kernel source files.  This is set 
#  in the makefile.  Warns if the kernel header version does not match
#  the running kernel version.
#  KVER = the version of the kernel headers
#  RVER = the version of the running kernel
if [ -f .kver ]; then
    KVER=`sed -e 's/-SMP//' -e 's/-MOD//' .kver`
    KVERSION=`cat .kver` 
else
    KVER=`echo -e "#include <linux/version.h>\n _UTS_RELEASE UTS_RELEASE" | \
    /lib/cpp -I ${LINUX_SRC}/include | \
    grep UTS_RELEASE | \
    awk '{print $2}' | tr -d \"`
    KVERSION=$KVER
    if grep -q "#define CONFIG_MODVERSIONS" \
			$LINUX_SRC/include/linux/autoconf.h ; then 
	KVERSION=${KVERSION}-MOD
    fi
    if grep -q smp_ $LINUX_SRC/include/linux/modules/ksyms.ver ; then
	KVERSION=${KVERSION}-SMP
    fi
fi
if grep -q smp_ /proc/ksyms ; then 
	RSMP='-SMP'
fi
if grep -q "kfree" /proc/ksyms &&
   ! grep -q "kfree$$" /proc/ksyms ; then
	RMOD='-MOD'
fi
#RVERSION=`uname -r`$RMOD$RSMP
RVERSION=`uname -r`
echo " "
echo "Installing Rocketport driver for Linux $KVERSION"
echo " "
if [ $KVERSION != $RVERSION ]; then
    echo "WARNING: The current kernel is actually version $RVERSION."
    echo " "
fi

#
# Actually install the kernel module
#
if [ ! -d /lib/modules/$KVER/misc ]; then 
	mkdir -p /lib/modules/$KVER/misc
fi

cp $OBJS /lib/modules/$KVER/misc

#
# The mod_rocket.o file is no longer used, so remove it to avoid confusion.
# Also, no longer putting rocket.o in /lib/modules, only /lib/modules/<version>
# 
rm -f /lib/modules/mod_rocket.o /lib/modules/$KVER/misc/mod_rocket.o
rm -f /lib/modules/rocket.o 

#
# OK, now we install the installation script 
#
if [ -d /etc/rc.d/init.d ]; then 
	# This is a RedHat or other system using a System V init scheme

        # Remove old-style rc.script
        rm -f /etc/rc.d/init.d/rc.rocket /etc/rc.d/rc?.d/[SK]??rc.rocket
	if [ ! -f /etc/rc.d/init.d/rocket ]; then
		echo "You are using a system which uses the System V init"
		echo "scheme.  The initialization script has been installed"
		echo "in /etc/rc.d/init.d and it has been automatically "
		echo "set up to be run when you reboot"

		ln -sf ../init.d/rocket /etc/rc.d/rc2.d/S83rocket
		ln -sf ../init.d/rocket /etc/rc.d/rc3.d/S83rocket
		ln -sf ../init.d/rocket /etc/rc.d/rc5.d/S83rocket
		ln -sf ../init.d/rocket /etc/rc.d/rc0.d/K18rocket
		ln -sf ../init.d/rocket /etc/rc.d/rc1.d/K18rocket
		ln -sf ../init.d/rocket /etc/rc.d/rc6.d/K18rocket
	fi
	if [ -f /etc/rc.d/rc.rocket ]; then
		echo "You have an old /etc/rc.d/rc.rocket script"
		echo "After checking to make sure it is no longer being used"
		echo "you should delete it."
	fi
	cp rc.rocket /etc/rc.d/init.d/rocket
	chmod 755 /etc/rc.d/init.d/rocket
elif [ -d /etc/init.d/rc3.d ]; then 
	# This is a SuSE system

        # Remove old-style rc.script
        rm -f /etc/init.d/rc.rocket /etc/init.d/rc?.d/[SK]??rc.rocket
	if [ ! -f /etc/init.d/rocket ]; then
		echo "You are using a system which uses the SuSE "
		echo "System V init directory layout.  The initialization"
		echo "script has been installed in /etc/init.d and it"
		echo "has been automatically set up to be run when you reboot"

		ln -sf ../rocket /etc/init.d/rc2.d/S83rocket
		ln -sf ../rocket /etc/init.d/rc3.d/S83rocket
		ln -sf ../rocket /etc/init.d/rc5.d/S83rocket
		ln -sf ../rocket /etc/init.d/rc0.d/K20rocket
		ln -sf ../rocket /etc/init.d/rc1.d/K20rocket
		ln -sf ../rocket /etc/init.d/rc6.d/K20rocket
	fi
	cp rc.rocket /etc/init.d/rocket
	chmod 755 /etc/init.d/rocket
elif [ -d /etc/init.d ]; then 
	# This is a Debian system

        # Remove old-style rc.script
        rm -f /etc/init.d/rc.rocket /etc/rc?.d/[SK]??rc.rocket
	if [ ! -f /etc/init.d/rocket ]; then
		echo "You are using a system which uses the Debian "
		echo "System V init directory layout.  The initialization"
		echo "script has been installed in /etc/init.d and it"
		echo "has been automatically set up to be run when you reboot"

		ln -sf ../init.d/rocket /etc/rc2.d/S83rocket
		ln -sf ../init.d/rocket /etc/rc3.d/S83rocket
		ln -sf ../init.d/rocket /etc/rc5.d/S83rocket
		ln -sf ../init.d/rocket /etc/rc0.d/K20rocket
		ln -sf ../init.d/rocket /etc/rc1.d/K20rocket
		ln -sf ../init.d/rocket /etc/rc6.d/K20rocket
	fi
	cp rc.rocket /etc/init.d/rocket
	chmod 755 /etc/init.d/rocket
else
        # This must be a Slackware or other non-SysV init system
	if [ ! -f /etc/rc.d/rc.rocket ]; then
		echo "The initialization script has been installed in "
		echo "/etc/rc.d/rc.rocket.  You will need to edit your "
		echo "/etc/rc files to run /etc/rc.d/rc.rocket"

		if [ ! -d /etc/rc.d ]; then 
			mkdir -p /etc/rc.d
		fi

		cp rc.rocket /etc/rc.d
	fi
fi

#
# Copy in the rocketport.conf file, if isn't already installed
#
if [ ! -f /etc/rocketport.conf ]; then
	cp rocketport.conf /etc
	echo "If running ISA cards, make sure you check and edit"
	echo "the /etc/rocketport.conf file!"
fi

#
# Install the setrocket program
#
cp setrocket /bin/setrocket
strip /bin/setrocket
chmod 755 /bin/setrocket
#
# Install the loadrm2 program
#
cp loadrm2/loadrm2 /sbin/loadrm2
strip /sbin/loadrm2
chmod 755 /sbin/loadrm2
cp loadrm2/ctmmdmld.rm loadrm2/ctmmdmfw.rm /etc
chmod 644 /etc/ctmmdmld.rm /etc/ctmmdmfw.rm

